function output = WaTMRI(input,varargin)   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% minimize alpha*TV(y)+lambda*0.5*||G*Phi*y-z||_2^2+beta*(||z||_{2,1}+ ||Phi'*y||_1)
%%% + 0.5*||Ry-b||_2^2 
% input:
%          Phi: wavelet transform
%          Phi': inverse wavelet transform
%          R: partial Fourier transfrom
%          b: measurements
%          G:  binary matrix to transfer overlap groups to non-overlap
%             groups by extending x to z with G*x=z, where x,z are wavelet
%             coefficients        
%          groups: groups index of each element in z
%          beta,lambda,alpha: parameters

%%% Written by Chen Chen and Junzhou Huang at UT Arlington
%%% April. 13, 2012

%%% Related paper: Chen Chen and Junzhou Huang, "Compressive Sensing MRI
%%% with Wavelet Tree Sparsity", In Proc. of the 26th Annual Conference on
%%% Neural Information Processing Systems (NIPS), Nevada, USA, December 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parsin.MAXITER=1; parsin.tv='l1';

l=input.l; u=input.u;
if((l==-Inf)&&(u==Inf))
    project=@(x)x;
elseif (isfinite(l)&&(u==Inf))
    project=@(x)(((l<x).*x)+(l*(x<=l)));
elseif (isfinite(u)&&(l==-Inf))
     project=@(x)(((x<u).*x)+((x>=u)*u));
elseif ((isfinite(u)&&isfinite(l))&&(l<u))
    project=@(x)(((l<x)&(x<u)).*x)+((x>=u)*u)+(l*(x<=l));
else
    error('lower and upper bound l,u should satisfy l<u');
end

t00 = cputime; 
n1=input.n1; n2=input.n2; N=n1*n2;
A=input.A; Phi=input.Phi;b=input.b; Atb=A'*b;

Lx=input.L; tnew=1;
alpha = input.alpha;
beta = input.beta; 
lambda = input.lamda;
G=input.G;GtG=input.GtG; 
groups=input.groups; Gmat=input.Gmat;

y=zeros(input.n1*input.n2,1);
yr=y;
for itr = 1:input.maxitr  % total iter counter        

    told=tnew;
    yp=y;
    
    if itr==1
        z = zeros(size(G,1),1);
    else
       z=shrink_group(G*xg,beta/lambda,0,groups,Gmat);
    end

    yg=yr-(A'*(A*yr)-Atb)/Lx-(lambda/Lx)*(Phi'*(GtG*(Phi*yr))-Phi'*(G'*z));    
    
     if (itr==1)
        [yt, P]=denoise_TV_One(yg, 2*alpha/Lx,-inf,inf,[],parsin); 
    else
        [yt, P]=denoise_TV_One(yg, 2*alpha/Lx,-inf,inf,P,parsin);
    end
    
    xg=Phi*yg;
    x = shrink1D(xg,zeros(N,1), 2*beta/Lx);

    y=(Phi'* x+yt)/2.0;
    y = project(y);

 
    tnew=(1+sqrt(1+4*told^2))/2; 
    yr=y+((told-1)/tnew)*(y-yp);     
% 
    if itr>30 tnew=1; end    

    output.snr(itr)=snr(y, input.f);
    output.xtime(itr)=cputime-t00; 
    
     if itr>=input.no  
        break
     end
end

output.y=y; 

    function s1 = shrink1D(p,q,s)
        % shrink p toward q by up to s; p and q are vectors, s is scalar
        tmp = p - q; s1 = q + sign(tmp).*max(0,abs(tmp)-s);
    end
end